#!/bin/bash

# Copyright (c) 2012, 2016, Oracle and/or its affiliates. All rights reserved.

# The MIT License (MIT)

# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:

# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.

# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.


THIS_DIR=`/usr/bin/dirname ${BASH_SOURCE}`

trap "echo INSTALLATION FAILED" ERR
set -e
set -E
set -o pipefail

echo "Reading configuration"

source $THIS_DIR/../common/knobs

source $MANAGEABLE_INSTALL_TOOL_DIR/common/mik_lib

source $MANAGEABLE_INSTALL_TOOL_DIR/common/media_map

echo "Configuration read"

#-------------------------------------------------------------------------------
# Collect media

echo "Collecting media"
mik_collect_media FIREFOX
mik_collect_media SELENIUM
echo "Media collected"

#--------------------------------------------------------------------------------
# Install Firefox


FIREFOX_HOME=$FIREFOX_MEDIA_DOWNLOAD_DIR/firefox
if [ ! -f $FIREFOX_HOME/firefox ]
then
  echo "Installing Firefox"
  cd $FIREFOX_MEDIA_DOWNLOAD_DIR;
  /bin/tar -xjf $MEDIA_LOC_FIREFOX
  echo "Firefox installed"
else
  echo "Firefox already installed"
fi
PATH=$FIREFOX_HOME:$PATH

#--------------------------------------------------------------------------------
# Install Selenium
SELENIUM_HOME=$SELENIUM_MEDIA_DOWNLOAD_DIR
SELENIUM_CLASSPATH=$SELENIUM_HOME/selenium-server-standalone-2.48.2.jar

#--------------------------------------------------------------------------------
# Set Http Proxy
export http_proxy="http://$HTTP_PROXY_HOST:$HTTP_PROXY_PORT"
export https_proxy="https://$HTTPS_PROXY_HOST:$HTTPS_PROXY_PORT"


#--------------------------------------------------------------------------------
# Loading reference data into CPQ

echo "Loading Comms reference data into CPQ"

echo "Prepare CPQ-Comms Reference Data"
rm -fr $MANAGEABLE_INSTALL_TOOL_DIR/$IMPORT_REFERENCE_SCRIPT_NAME/data/cpq
mkdir -p $MANAGEABLE_INSTALL_TOOL_DIR/$IMPORT_REFERENCE_SCRIPT_NAME/data/cpq
/usr/bin/unzip -o -q -d $MANAGEABLE_INSTALL_TOOL_DIR/$IMPORT_REFERENCE_SCRIPT_NAME/data/cpq $MANAGEABLE_INSTALL_TOOL_DIR/$IMPORT_REFERENCE_SCRIPT_NAME/data/CommCarrierEthernet_Data.zip

echo "Establishing XServer"
/bin/bash -c "SUDO_ASKPASS=${MANAGEABLE_INSTALL_TOOL_DIR}/common/askpass_sudo.sh ; export SUDO_ASKPASS ; $SUDO_PROGRAM -A -k -E -p '%H:%p' /bin/kill Xvfb ; ${SUDO_PROGRAM} -A -k -E -b -p '%H:%p' /usr/bin/Xvfb :6046"
export DISPLAY=:6046
echo "XServer established"



#--------------------------------------------------------------------------------
# Validate Product Synch for PARTS

echo "Prepare get parts request"
cat $MANAGEABLE_INSTALL_TOOL_DIR/$IMPORT_REFERENCE_SCRIPT_NAME/importrefdata/cpq_getparts.xml.template | sed -e "s#CPQ_ADMIN_USER_NAME#$CPQ_ADMIN_USER_NAME#" -e "s#CPQ_ADMIN_USER_PASSWORD#$CPQ_ADMIN_USER_PASSWORD#" > $MANAGEABLE_INSTALL_TOOL_DIR/$IMPORT_REFERENCE_SCRIPT_NAME/importrefdata/cpq_getparts.xml
echo "Retrieve Parts .."
/usr/bin/curl -X POST -H "Content-Type: text/xml;charset=UTF-8" --data-binary \@$MANAGEABLE_INSTALL_TOOL_DIR/$IMPORT_REFERENCE_SCRIPT_NAME/importrefdata/cpq_getparts.xml  $CPQ_URL/v2_0/receiver/parts -o $MANAGEABLE_INSTALL_TOOL_DIR/$IMPORT_REFERENCE_SCRIPT_NAME/importrefdata/cpq_getparts_response.xml

# Check the Parts response for number of parts synched
NUMBER_OF_SYNCHED_PARTS=`xml_grep 'bm:records_returned' $MANAGEABLE_INSTALL_TOOL_DIR/$IMPORT_REFERENCE_SCRIPT_NAME/importrefdata/cpq_getparts_response.xml | grep -m 1 '<bm:records_returned' | cut -d '>' -f2 | cut -d '<' -f1`

if [ $NUMBER_OF_SYNCHED_PARTS == "0" ]; then
	echo "ERROR: Product Synch. Parts Table contains $NUMBER_OF_SYNCHED_PARTS parts. Please ensure product synch is successfully completed."
	exit 1
else
	echo
	echo "Number of parts retrieved from Parts table = $NUMBER_OF_SYNCHED_PARTS. Products synchronization validation done. Proceeding ..."
	echo
fi

# Get parts from template
NUMBER_OF_TEMPLATE_PRODS=$(xmllint --format $MANAGEABLE_INSTALL_TOOL_DIR/$IMPORT_REFERENCE_SCRIPT_NAME/importrefdata/cpq_parts_list.xml | grep -c  '<bm:part_number')
for i in $(seq 1 $NUMBER_OF_TEMPLATE_PRODS)
do
   XPATH_TO_PRODNAME="//bm:each_record[$i]/bm:part_number/text()"
   PartsFromTemplate+=$(/usr/bin/xpath $MANAGEABLE_INSTALL_TOOL_DIR/$IMPORT_REFERENCE_SCRIPT_NAME/importrefdata/cpq_parts_list.xml $XPATH_TO_PRODNAME)"|"
done

#Check if too fewer parts were synchronized
if [ "$NUMBER_OF_SYNCHED_PARTS" -lt "$NUMBER_OF_TEMPLATE_PRODS" ]; then

	echo "ERROR: Product Synch for Parts. Number of parts synchronized [$NUMBER_OF_SYNCHED_PARTS] lesser than number of expected number of parts [$NUMBER_OF_TEMPLATE_PRODS]."
	exit 1
	
else
	echo "Products synchronization validation of expected set of products list done. Proceeding ..."
	echo
fi

#--------------------------------------------------------------------------------
# Verify Product Synch for PARTS

# Get parts from Parts table
NUMBER_OF_PARTS=$(xmllint --format $MANAGEABLE_INSTALL_TOOL_DIR/$IMPORT_REFERENCE_SCRIPT_NAME/importrefdata/cpq_getparts_response.xml | grep -c '<bm:part_number')
for i in $(seq 1 $NUMBER_OF_PARTS)
do
   XPATH_TO_PARTNAME="//bm:each_record[$i]/bm:part_number/text()"
   PartsAfterProductSync+=$(/usr/bin/xpath $MANAGEABLE_INSTALL_TOOL_DIR/$IMPORT_REFERENCE_SCRIPT_NAME/importrefdata/cpq_getparts_response.xml $XPATH_TO_PARTNAME)"|"
done


OIFS=$IFS;
IFS="|";

PartsAfterProductSyncArray=($PartsAfterProductSync);
PartsTemplateArray=($PartsFromTemplate);


ProductsSynched=()
for ((i = 0; i < ${#PartsAfterProductSyncArray[@]}; i++)); do
    skip=
    for ((j = 0; j < ${#PartsTemplateArray[@]}; j++)); do
        [[ ${PartsAfterProductSyncArray[$i]} == ${PartsTemplateArray[$j]} ]] && { skip=1; break; }
    done
    [[ -n $skip ]] || ProductsSynched+=("${PartsAfterProductSyncArray[$i]}")
done

# Display the list of unsynchronized parts
for ((i = 0; i < ${#ProductsSynched[@]}; i++))
do
    echo "INFO: Non-synchronized product [${ProductsSynched[$i]}]"
done

#Check if extra set of parts were synchronized
NUMBER_OF_SYNCHED_PARTS=${#ProductsSynched[@]}
if [ $NUMBER_OF_SYNCHED_PARTS == "0" ]; then
	echo "Products synchronization verification done. Proceeding ..."
	echo
else
	echo "Product synch verification completed in parts table. Proceeding ..."
	echo
fi



IFS=$OIFS;


#--------------------------------------------------------------------------------
# Enrich PriceList 

echo "Enrich OC_PRICELIST Data table started in $CPQ_URL"

# Fetch EBC Bandwidth product
cat $MANAGEABLE_INSTALL_TOOL_DIR/$IMPORT_REFERENCE_SCRIPT_NAME/importrefdata/cpq_getpricelist.xml.template | sed -e "s#CPQ_ADMIN_USER_NAME#$CPQ_ADMIN_USER_NAME#" -e "s#CPQ_ADMIN_USER_PASSWORD#$CPQ_ADMIN_USER_PASSWORD#" > $MANAGEABLE_INSTALL_TOOL_DIR/$IMPORT_REFERENCE_SCRIPT_NAME/importrefdata/cpq_getpricelist.xml
echo "Fetch from Datable OC_PRICELIST in $CPQ_URL"
/usr/bin/curl -X POST -H "Content-Type: text/xml;charset=UTF-8" --data-binary \@$MANAGEABLE_INSTALL_TOOL_DIR/$IMPORT_REFERENCE_SCRIPT_NAME/importrefdata/cpq_getpricelist.xml  $CPQ_URL/v2_0/receiver/dataTables -o $MANAGEABLE_INSTALL_TOOL_DIR/$IMPORT_REFERENCE_SCRIPT_NAME/importrefdata/cpq_getpricelist_response.xml
sleep 2

# Parse the rate tier id for EBC Bandwidth product
RATE_TIER_ID=`xml_grep 'bm:rateTierId' $MANAGEABLE_INSTALL_TOOL_DIR/$IMPORT_REFERENCE_SCRIPT_NAME/importrefdata/cpq_getpricelist_response.xml | grep -m 1 '<bm:rateTierId' | cut -d '>' -f2 | cut -d '<' -f1`
echo "Rate Tier ID = [$RATE_TIER_ID]"

cat $MANAGEABLE_INSTALL_TOOL_DIR/$IMPORT_REFERENCE_SCRIPT_NAME/importrefdata/cpq_updatepricelist.xml.template | sed -e "s#CPQ_ADMIN_USER_NAME#$CPQ_ADMIN_USER_NAME#" -e "s#CPQ_ADMIN_USER_PASSWORD#$CPQ_ADMIN_USER_PASSWORD#"  -e "s#RATE_TIER_ID#$RATE_TIER_ID#" > $MANAGEABLE_INSTALL_TOOL_DIR/$IMPORT_REFERENCE_SCRIPT_NAME/importrefdata/cpq_updatepricelist.xml
echo "Updating Data table OC_PRICELIST in $CPQ_URL"
/usr/bin/curl -X POST -H "Content-Type: text/xml;charset=UTF-8" --data-binary \@$MANAGEABLE_INSTALL_TOOL_DIR/$IMPORT_REFERENCE_SCRIPT_NAME/importrefdata/cpq_updatepricelist.xml  $CPQ_URL/v2_0/receiver/dataTables -o $MANAGEABLE_INSTALL_TOOL_DIR/$IMPORT_REFERENCE_SCRIPT_NAME/importrefdata/cpq_updatepricelist_response.xml
sleep 2


# Set Data table name for deployment
tablename="OC_PRICELIST"

# Deploy OC_PRICELIST Data table
cat $MANAGEABLE_INSTALL_TOOL_DIR/$IMPORT_REFERENCE_SCRIPT_NAME/importrefdata/cpq_deploy_datatable.xml.template | sed -e "s#CPQ_ADMIN_USER_NAME#$CPQ_ADMIN_USER_NAME#" -e "s#CPQ_ADMIN_USER_PASSWORD#$CPQ_ADMIN_USER_PASSWORD#" -e "s#TABLE_NAME#$tablename#"> $MANAGEABLE_INSTALL_TOOL_DIR/$IMPORT_REFERENCE_SCRIPT_NAME/importrefdata/cpq_deploy_$tablename.xml
echo "Deploying DataTable $tablename into $CPQ_URL"
/usr/bin/curl -X POST -H "Content-Type: text/xml;charset=UTF-8" --data-binary \@$MANAGEABLE_INSTALL_TOOL_DIR/$IMPORT_REFERENCE_SCRIPT_NAME/importrefdata/cpq_deploy_$tablename.xml  $CPQ_URL/v2_0/receiver/dataTables -o $MANAGEABLE_INSTALL_TOOL_DIR/$IMPORT_REFERENCE_SCRIPT_NAME/importrefdata/cpq_deploy_${tablename}_response.xml
sleep 2



prefix=$MANAGEABLE_INSTALL_TOOL_DIR/$IMPORT_REFERENCE_SCRIPT_NAME/data/cpq/CommCarrierEthernet_
echo "Set Prefix to $prefix"

for tname in $(ls $MANAGEABLE_INSTALL_TOOL_DIR/$IMPORT_REFERENCE_SCRIPT_NAME/data/cpq/*_OC_*zip  | rev | cut -d"." -f2 | rev)
do
    tablename=${tname#$prefix}
    echo "Loading DataTable $tablename into $CPQ_URL"

    # extract the data table xml file with pretty print format
    /bin/rm -rf /tmp/$tablename
    /bin/mkdir -p /tmp/$tablename
    /usr/bin/unzip -o -q -d /tmp/$tablename $MANAGEABLE_INSTALL_TOOL_DIR/$IMPORT_REFERENCE_SCRIPT_NAME/data/cpq/CommCarrierEthernet_$tablename.zip

    # remove the redundant <id/> tags
    /usr/bin/xsltproc $MANAGEABLE_INSTALL_TOOL_DIR/$IMPORT_REFERENCE_SCRIPT_NAME/importrefdata/remove_ids.xsl /tmp/$tablename/*.xml > $MANAGEABLE_INSTALL_TOOL_DIR/$IMPORT_REFERENCE_SCRIPT_NAME/data/cpq/$tablename.xml
    # zip the edited xml for upload
    /usr/bin/zip -j $MANAGEABLE_INSTALL_TOOL_DIR/$IMPORT_REFERENCE_SCRIPT_NAME/data/cpq/$tablename.zip $MANAGEABLE_INSTALL_TOOL_DIR/$IMPORT_REFERENCE_SCRIPT_NAME/data/cpq/$tablename.xml

    $JAVA_HOME/bin/java -DcpqUrl=$CPQ_URL -Dusername=$CPQ_ADMIN_USER_NAME -Dpassword=$CPQ_ADMIN_USER_PASSWORD -Dseleniumtimeout=$CPQ_SELENIUM_TIMEOUT -Dautoconfig_url=$CPQ_AUTOCONFIG_URL -Dfile=$MANAGEABLE_INSTALL_TOOL_DIR/$IMPORT_REFERENCE_SCRIPT_NAME/data/cpq/$tablename.zip -cp $MANAGEABLE_INSTALL_TOOL_DIR/$IMPORT_REFERENCE_SCRIPT_NAME/data/cpq:$SELENIUM_CLASSPATH -enableassertions oracle.communications.cpq.automation.CpqUploadData
    if [ $? != 0 ]
    then
       echo "Creation of $tablename in $CPQ_URL resulted in ERRORS"
       exit 1
    else
    echo "Loaded $tablename in $CPQ_URL"
    fi
    sleep 5

    # Deploy Data Table
    cat $MANAGEABLE_INSTALL_TOOL_DIR/$IMPORT_REFERENCE_SCRIPT_NAME/importrefdata/cpq_deploy_datatable.xml.template | sed -e "s#CPQ_ADMIN_USER_NAME#$CPQ_ADMIN_USER_NAME#" -e "s#CPQ_ADMIN_USER_PASSWORD#$CPQ_ADMIN_USER_PASSWORD#" -e "s#TABLE_NAME#$tablename#"> $MANAGEABLE_INSTALL_TOOL_DIR/$IMPORT_REFERENCE_SCRIPT_NAME/importrefdata/cpq_deploy_$tablename.xml
    echo "Deploying DataTable $tablename into $CPQ_URL"
    /usr/bin/curl -X POST -H "Content-Type: text/xml;charset=UTF-8" --data-binary \@$MANAGEABLE_INSTALL_TOOL_DIR/$IMPORT_REFERENCE_SCRIPT_NAME/importrefdata/cpq_deploy_$tablename.xml  $CPQ_URL/v2_0/receiver/dataTables -o $MANAGEABLE_INSTALL_TOOL_DIR/$IMPORT_REFERENCE_SCRIPT_NAME/importrefdata/cpq_deploy_${tablename}_response.xml
    sleep 2

done



#--------------------------------------------------------------------------------
# Load Parts

echo "Loading Parts into $CPQ_URL"
$JAVA_HOME/bin/java -DcpqUrl=$CPQ_URL -Dusername=$CPQ_ADMIN_USER_NAME -Dpassword=$CPQ_ADMIN_USER_PASSWORD -Dseleniumtimeout=$CPQ_SELENIUM_TIMEOUT -Dautoconfig_url=$CPQ_AUTOCONFIG_URL -Dfile=$MANAGEABLE_INSTALL_TOOL_DIR/$IMPORT_REFERENCE_SCRIPT_NAME/data/cpq/CommCarrierEthernet_Parts.zip -cp $MANAGEABLE_INSTALL_TOOL_DIR/$IMPORT_REFERENCE_SCRIPT_NAME/data/cpq:$SELENIUM_CLASSPATH -enableassertions oracle.communications.cpq.automation.CpqUploadData
if [ $? != 0 ]
then
  echo "Creation of Parts in $CPQ_URL resulted in ERRORS"
  exit 1
else
  echo "Loaded Parts in $CPQ_URL"
fi
sleep 8



#--------------------------------------------------------------------------------
echo "Enrich Parts in CPQ that were synched from PDC"

# Replace credentials in template and generate request payload
cat $MANAGEABLE_INSTALL_TOOL_DIR/$IMPORT_REFERENCE_SCRIPT_NAME/importrefdata/cpq_updateparts.xml.template | sed -e "s#CPQ_ADMIN_USER_NAME#$CPQ_ADMIN_USER_NAME#" -e "s#CPQ_ADMIN_USER_PASSWORD#$CPQ_ADMIN_USER_PASSWORD#" > $MANAGEABLE_INSTALL_TOOL_DIR/$IMPORT_REFERENCE_SCRIPT_NAME/importrefdata/cpq_updateparts.xml

# Find all parts records to be replaced in the parts list 
xml_grep 'bm:each_record' $MANAGEABLE_INSTALL_TOOL_DIR/$IMPORT_REFERENCE_SCRIPT_NAME/importrefdata/cpq_parts_list.xml | sed -n -e '/file/,/\/file/ p' | grep -v file > $MANAGEABLE_INSTALL_TOOL_DIR/$IMPORT_REFERENCE_SCRIPT_NAME/importrefdata/cpq_updatepart.xml.fragment

# Insert the part records in the request payload
sed -i "/<bm:Parts>/r $MANAGEABLE_INSTALL_TOOL_DIR/$IMPORT_REFERENCE_SCRIPT_NAME/importrefdata/cpq_updatepart.xml.fragment" $MANAGEABLE_INSTALL_TOOL_DIR/$IMPORT_REFERENCE_SCRIPT_NAME/importrefdata/cpq_updateparts.xml

# Remove temporary files
/bin/rm -f $MANAGEABLE_INSTALL_TOOL_DIR/$IMPORT_REFERENCE_SCRIPT_NAME/importrefdata/cpq_updatepart.xml.fragment

# Post the update parts request payload and receive response
/usr/bin/curl -X POST -H "Content-Type: text/xml;charset=UTF-8" --data-binary \@$MANAGEABLE_INSTALL_TOOL_DIR/$IMPORT_REFERENCE_SCRIPT_NAME/importrefdata/cpq_updateparts.xml  $CPQ_URL/v2_0/receiver/parts -o $MANAGEABLE_INSTALL_TOOL_DIR/$IMPORT_REFERENCE_SCRIPT_NAME/importrefdata/cpq_updateparts_response.xml

# Check the update parts response for the number of parts that failed enrichment
NUMBER_OF_PARTS_FAIL_ENRICH=`xml_grep 'bm:status' $MANAGEABLE_INSTALL_TOOL_DIR/$IMPORT_REFERENCE_SCRIPT_NAME/importrefdata/cpq_updateparts_response.xml | grep -m 1 '<bm:records_failed' | cut -d '>' -f2 | cut -d '<' -f1`

if [ $NUMBER_OF_PARTS_FAIL_ENRICH == "0" ]; then
     echo
     echo "Parts enrichment done."
     echo
else
     echo "ERROR: Failed to enrich $NUMBER_OF_PARTS_FAIL_ENRICH parts."
     exit 1
fi

IFS=$OIFS;


#--------------------------------------------------------------------------------
echo "Completed loading CPQ Comms reference data "

